----------------------------------------------------------------------------------------------------------------------------------------------------
--	Script: Unwrap tools 1.50
--	Version: 1.50
--	tested using Max 6.0, Max 7.0, Max 8.0 , max 9.0
----------------------------------------------------------------------------------------------------------------------------------------------------
--	Author: Diego Garcia Huerta
--	Originally written by Steve Johnson 
--	Email: diegogh@sloft.com
--	Created: 10 November 2004
----------------------------------------------------------------------------------------------------------------------------------------------------
--	Modified: 															
--	
--		24 november 2004
--		- fit relax
--		- relax 2
--		- soft selection
--		- options from max7 : edge loop, paint mode
--		- new about rollout
--	   17 january 2006
--		- added expand/compact uvs
--	   15 may 2007
--		- added max9 missing quad menus 
----------------------------------------------------------------------------------------------------------------------------------------------------
--	Required Files:														
--		uvtools_24i.tga														
----------------------------------------------------------------------------------------------------------------------------------------------------
--	Description:														
--		Taken originally from www.chuggnut.com , I have modified it to code some of the new tools included in Max7 as well as the 
--		 original soft selection tools that were not included in this version of Unrwap tools. I've made some changes, fixed some bugs and 
--		commented the code to make it more readable.
----------------------------------------------------------------------------------------------------------------------------------------------------
--	Requested by: 														
--		Juan Solis, Diego Garcia Huerta															
----------------------------------------------------------------------------------------------------------------------------------------------------
--	 1996-2007 www.sloft.com
----------------------------------------------------------------------------------------------------------------------------------------------------

--==================================================================================================================================================
-- global variables
--==================================================================================================================================================
Global unwrapholder, dock

--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
--	Rollout:
--		About rollout 
--
--	Description:
--		About rollout of the Unwrap Tools application.
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
rollout uiUnwrapholderAboutRollout "About ..." width:265 height:135
(
	local clWindowText 			= (colorman.getcolor #windowtext * 255) as color
	local cl3dDarkShadow 		= (colorman.getcolor #3ddarkshadow * 255) as color
	local clRollupTitleText 	= (colorman.getcolor #rollupTitleText * 255) as color
	local clRollupTitleFace 	= (colorman.getcolor #rollupTitleFace * 255) as color
	local clRollupTitleShadow 	= (colorman.getcolor #rollupTitleShadow * 255) as color
	local nUnwrapUIsizeX 	= 275 -- width of the about rollout
	local nUnwrapUIsizeY 	= 135 -- height of the about rollout
	
	GroupBox 	uiAboutGroup	 	"" 								pos:[8,1] 		width:(nUnwrapUIsizeX - 30) height:(nUnwrapUIsizeY - 10)
	ColorPicker uiAboutRollColor 	""								pos:[4,7] 		width:(nUnwrapUIsizeX - 26) height:22 enabled:false 	color:clRollupTitleFace 
	HyperLink 	uiLblPlugin 		"Unwrap Tools MaxScript v1.50" 	pos:[20, 12]  	address:"http://www.sloft.com/MaxScripts/UnwrapExt/"	color:clRollupTitleShadow  hoverColor:clWindowText visitedColor:clRollupTitleText
	HyperLink 	uiLblAuthor 		"Diego Garcia Huerta" 			pos:[20, 40] 	address:"mailto:diegogh@sloft.com" 					color:cl3dDarkShadow 	hoverColor:clWindowText visitedColor:cl3dDarkShadow 
	HyperLink 	uiLblDate			"www.sloft.com ( May 2007 )" 	pos:[20,60] 	address:"http://www.sloft.com/" 					color:clRollupTitleShadow  hoverColor:clWindowText visitedColor:clRollupTitleText
	HyperLink 	uiLblAuthor2 		"Originally written by Steve Johnson" 			pos:[20, 85] 	address:"mail@chuggnut.com" 		color:cl3dDarkShadow 	hoverColor:clWindowText visitedColor:cl3dDarkShadow 
	HyperLink 	uiLblDate2			"www.chuggnut.com ( 2003 )" 	pos:[20,105] 	address:"http://www.chuggnut.com/" 					color:clRollupTitleShadow  hoverColor:clWindowText visitedColor:clRollupTitleText
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-


--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
--	Rollout:
--		unwrapholder rollout 
--
--	Description:
--		Unrwap tools rollout
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
rollout unwrapholder "Unwrap tools" width:130 height:20
(
	--==================================================================================================================================================
	-- unwrapholder  Variables
	--==================================================================================================================================================
	local icn 			 = 54	 -- number of icons in the icons file for the UI
	local hidestate 	 = false -- state of the unwrapholder panel
	local versionno 	 = 1.50	 -- version of this script
	local subobjectstate = 1	 -- subobject mode state, 1 = vertex, 2 = edge, 3 = face
	local UV 			 = modpanel.getcurrentobject()				-- this is the variable that contains the current modifier on the panel
	local uviconsI 		 = (getdir #ui + "/icons/uvtools_24i.bmp")	-- file where to obtain the icons
	struct dims (width, height, mid, maxUV, minUV)					-- struct used to return the size of the element selected in the Unwrap window

	--==================================================================================================================================================
	-- unwrapholder  functions
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function getDims sel &uv 
	--
	--	Remarks:
	-- 		calculates the boundingbox size of the selected verts in the Unwrap window, of the vertex selection
	--	Parameters:
	--		sel: array containing the id of the verts selected
	--		&uv: unwrap modifier
	--	Return Value:
	--		returns a dim struct containing the size of the selection 
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function getDims sel &uv =
	(
		local minUV = [99999,99999,0]
		local maxUV = [-99999,-99999,0]
		local mid 	= [0,0,0]
		
		for i in sel do
		(
			local p = uv.getvertexposition currenttime i	
				
			--check X
			if (p.x > maxUV.x) then
			(
				maxUV.x = p.x
			)
			else
			(
				if (p.x < minUV.x) do
					minUV.x = p.x
			)
	
			--check Y
			if (p.y > maxUV.y) then
			(
				maxUV.y = p.y
			)
			else
			(
				if (p.y < minUV.y) do
					minUV.y = p.y
			)			
		)
		
		local w = (maxUV.x - minUV.x)
		local h = (maxUV.y - minUV.y)
		return (dims width:w height:h mid:([minUV.x + (w * 0.5), minUV.Y + (h * 0.5),0]) maxUV:maxuv minUV:minuv)
	)
	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function seltovertmode uvmod
	--
	--	Remarks:
	-- 		converts the selection to the subobject mode specified
	--	Parameters:
	--		uvmod:unwrap modifier. available modes 2: edge mode, 3: face mode
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function seltovertmode uvmod = 
	(
		case uvmod.gettvsubobjectmode() of
		(
			2: uvmod.edgetovertselect()
			3: uvmod.facetovertselect()
		)
	)		
	--==================================================================================================================================================
	-- unwrapholder User Interface
	--==================================================================================================================================================

	button showhide "Hide" 	pos:[2,2] 	width:35 height:15 tooltip:"Minimize"
	button dodock 	"<" 	pos:[89,2] 	width:15 height:15 tooltip:"Dock Side"
	button dohelp 	"?" 	pos:[105,2] width:15 height:15 tooltip:"About..."
	label inf1 		"V???" 	pos:[43,2] 	width:40 height:15
	
	subrollout 	holder width:125 height:300 align:#center
	checkButton subObjVert 	"" 	pos:[11,350] width:27 height:27 images:#(uviconsI, undefined, icn, 30, 27, 30, 27) checked:true tooltip:"<CTRL> converts the current selection to vertex selection"
	checkButton subObjEdge 	"" 	pos:[41,350] width:27 height:27 images:#(uviconsI, undefined, icn, 31, 28, 31, 28) tooltip:"<CTRL> converts the current selection to edge selection"
	checkButton subObjFace 	"" 	pos:[71,350] width:27 height:27 images:#(uviconsI, undefined, icn, 32, 29, 32, 29) tooltip:"<CTRL> converts the current selection to face selection"

	button 		dash_grow 	"+" pos:[101,16] width:15 height:13 toolTip:"Expand Selection"
	button 		dash_shrink "-" pos:[101,30] width:15 height:13 toolTip:"Contract Selection"

	checkbutton dash_PaintMode 		"" 	pos:[490-57,16] width:27 height:28 images:#(uviconsI, undefined, icn, 46, 51, 46, 51) checked: (UV.unwrap2.getPaintSelectMode()) toolTip:"Paint Select Mode" 
	button 		dash_PaintModeInc 	"+" pos:[520-57,16] width:14 height:14  toolTip:"Paint Brush Grow"
	button 		dash_PaintModeDec 	"-" pos:[520-57,30] width:14 height:14  toolTip:"Paint Brush Shrink"

	button dash_EdgeLoop "Edge Loop" 		pos:[520-40,16] width:58 height:18  toolTip:"Edge Loop Selection" enabled:(UV.getTVSubObjectMode() == 2)
	checkbox syncelement "Sync to Element" 	pos:[12,383] width:106 height:14 checked:(UV.getTVElementMode())
	checkbox softcel 	 "Soft Selection" 	pos:[22,383] width:106 height:14 checked:(UV.getenablesoftselection())

	--==================================================================================================================================================
	-- unwrapholder functions
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		fn updateUI 
	--
	--	Remarks:
	-- 		 updates the position of the UI elements depending if the panel is hidden or not
	--	Parameters:
	--		
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	fn updateUI =
	(
		-- updates the pos of the UI elements depending if the panel is hidden or not
		if (hidestate == false) then
		(
			h = uv.GetWindowH()
			unwrapholder.height = (h - 6)
			holder.height = (h - 130)
			
			subObjVert.pos = [13,(h - 103)]
			subObjEdge.pos = [43,(h - 103)]
			subObjFace.pos = [73,(h - 103)]
			dash_grow.pos = [103,h - 103] 
			dash_shrink.pos = [103, h - 89]

			dash_PaintMode.pos = [13,(h - 73)] 
			dash_PaintModeInc.pos = [43,(h - 73)]
			dash_PaintModeDec.pos = [43,(h - 59)]
			dash_EdgeLoop.pos = [60,(h - 69)] 

			syncelement.pos = [12,(h - 42)]
			softcel.pos = [12,(h - 23)]
		)
		else
		(
			unwrapholder.height = 20
		)

		unwrapMod = modpanel.getcurrentobject()

		-- updates the buttons that have to enagbled/disabled according to the subObject mode
		dash_EdgeLoop.enabled = (unwrapMod.getTVSubObjectMode() == 2)
	)

	--==================================================================================================================================================
	-- unwrapholder Events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on softcel changed state
	--
	--	Remarks:
	-- 		manages when the user changes the soft selection checkbox
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on softcel changed state do
	(
		UV.setenablesoftselection state
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dodock pressed
	--
	--	Remarks:
	-- 		manages when the user decides to change the dock from one side to the other
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dodock pressed do
	(
		xPKW = uv.GetWindowX()
		yPKW = uv.GetWindowY()
		hPKW = uv.GetWindowH()
		wPKW =  uv.GetWindowW() 		
		
		if (dock == #right) or (dock == undefined) then
		(
			dock = #left
			SetDialogPos unwrapholder [xpkw - 135, ypkw]	
			dodock.text = ">"
		)
		else
		(
			dock = #right
			SetDialogPos unwrapholder [xpkw + wpkw, ypkw]
			dodock.text = "<"
		)
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on subObjVert changed state
	--
	--	Remarks:
	-- 		manages when the user changes to vert mode. CTRL pressed gets the current selection and converts it to vert mode
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on subObjVert changed state do
	(
		if keyboard.controlpressed do
		(
			case subobjectstate of
			(
				2: uv.edgetovertselect()
				3: uv.facetovertselect()
			)
		)
		
		UV.setTVSubObjectMode 1
		subObjEdge.checked = false			
		subObjFace.checked = false
		subObjVert.checked = true
		subobjectstate = 1
		updateUI()
		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on subObjEdge changed state
	--
	--	Remarks:
	-- 		manages when the user changes to ende mode. CTRL pressed gets the current selection and converts it to edge mode
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on subObjEdge changed state do
	(
		if keyboard.controlpressed do
		(
			case subobjectstate of
			(
				1: uv.verttoedgeselect()
				3: uv.facetoedgeselect()
			)
		)
		
		UV.setTVSubObjectMode 2
		subObjVert.checked = false			
		subObjFace.checked = false
		subObjEdge.checked = true
		subobjectstate = 2
		updateUI()
		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on subObjFace changed state
	--
	--	Remarks:
	-- 		manages when the user changes to face mode. CTRL pressed gets the current selection and converts it to face mode
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on subObjFace changed state do
	(
		if keyboard.controlpressed do
		(
			case subobjectstate of
			(
				1: uv.verttofaceselect()
				2: uv.edgetofaceselect()
			)
		)
		
		UV.setTVSubObjectMode 3
		subObjVert.checked = false			
		subObjEdge.checked = false
		subObjFace.checked = true
		subobjectstate = 3
		updateUI()
	)	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on syncelement changed state
	--
	--	Remarks:
	-- 		sets the state of the Select Element checkbox in the Selection Mode group of the Editors panel
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on syncelement changed state do
	(
		UV.setTVElementMode state
	)	
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on showhide pressed
	--
	--	Remarks:
	-- 		manages when the user hide or show the main window of the unwrap tools script
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on showhide pressed do
	(
		if (hidestate == true) then
		(
			hidestate = false
			showhide.text = "Hide"
		)
		else
		(
			unwrapholder.height = 20
			hidestate = true
			showhide.text = "Show"
		)
		
		updateUI()
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_grow pressed do 
	--
	--	Remarks:
	-- 		expands the selection by one element
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_grow pressed do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.expandSelection()
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_shrink pressed do
	--
	--	Remarks:
	-- 		shrink the selection by one element
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_shrink pressed do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.contractSelection()
	)	
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_EdgeLoop pressed do
	--
	--	Remarks:
	-- 		Selects UV Edge Loops. Select any number of edges and call this method to select all edges that build an edge loop with the original 
	--		selection
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_EdgeLoop pressed do
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.uvEdgeSelect()

	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_PaintModeInc pressed do
	--
	--	Remarks:
	-- 		Increments the PaintSelect brush size by 1
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_PaintModeInc pressed do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.PaintSelectIncSize()
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_PaintModeDec pressed do
	--
	--	Remarks:
	-- 		Decrements the PaintSelect brush size by 1
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_PaintModeDec pressed do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.PaintSelectDecSize()
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_PaintMode changed state do
	--
	--	Remarks:
	-- 		Sets the PaintSelect Mode
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_PaintMode changed state do
	(
		unwrapMod = modpanel.getcurrentobject()
		
		if (state == false) do 
		(
			unwrapMod.setPaintSelectMode false
			unwrapMod.unwrap.move()
		)
		if state == true do 
		(
			unwrapMod.setPaintSelectMode true		
		)

	)

	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on unwrapholder open
	--
	--	Remarks:
	-- 		manages when the main rollout of the app is opened
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on unwrapholder open do
	(
		try
		(
			subobjectlevel = 1
		)
		catch()
		
		case (UV.getTVSubObjectMode()) of
		(
			1: (subObjVert.checked = true; subObjEdge.checked = false; subObjFace.checked = false; subobjectstate = 1)
			2: (subObjVert.checked = false; subObjEdge.checked = true; subObjFace.checked = false; subobjectstate = 2)
			3: (subObjVert.checked = false; subObjEdge.checked = false; subObjFace.checked = true; subobjectstate = 3)
		)

		-- setup correct panel size
		updateUI()

		hidestate = false
		showhide.text = "Hide"	
		inf1.text = "V" + versionno as string	

	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dohelp pressed
	--
	--	Remarks:
	-- 		about rollout
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dohelp pressed do
	(
		createdialog uiUnwrapholderAboutRollout 
	)	
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-












--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		Align Tools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout aligntools "Align Tools" width:114 height:45
(
	--==================================================================================================================================================
	-- aligntools variables
	--==================================================================================================================================================
	local uv = unwrapholder.uv
	
	--==================================================================================================================================================
	-- aligntools functions
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function centerpoint arr
	--
	--	Remarks:
	-- 		returns the average of the 3d points in the array
	--	Parameters:
	--
	--	Return Value:
	--		returns the average of the 3d points in the array
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function centerpoint arr = 
	(
		local mid = [0,0,0]
		for i = 1 to arr.count do
			mid += arr[i]
		
		return (mid / arr.count)
	)	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function propertysort p prop
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function propertysort p prop = 
	(
		for i = 1 to (p.count - 1) do
		(
			low = prop[i]
			pindex = i
			
			--find lowest in list
			for n = (i + 1) to p.count do
			(
				if (prop[n] < low) then
				(
					low = prop[n]
					pindex = n
				)
			)
			
			swap p[i] p[pindex]
 			swap prop[i] prop[pindex]
		)
		return p	
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function subarray arr start end
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function subarray arr start end = 
	(
		a = #()
		acount = 0
		
		for i = start to end do
		(
			if arr[i] == undefined then
				return undefined
				
			a[acount += 1] = arr[i]
		)
		return a
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function lineofbestfit p axis =
	(
		-- sort points into length order
		l = #()
		
		case axis of
		(
			#x: for i = 1 to p.count do (l[i] = p[i].x)
			#y: for i = 1 to p.count do (l[i] = p[i].y)
			#z: for i = 1 to p.count do (l[i] = p[i].z)
		)
	
		p = propertysort p l
		
		-- divide point set into 3, if uneven middle section gets odd number
		
		count = p.count / 3
		over = mod p.count 3
		
		a = subarray p 1 count
		b = subarray p (count + 1)(count * 2 + over)
		c = subarray p (count * 2 + over + 1) (count * 3 + over)
	
		-- find centre of each section (A,B,C)
		
		acent = centerpoint a
		bcent = centerpoint b
		ccent = centerpoint c
		
		-- plot line A,C
		--find length from B to line
		
		dline = (ccent - acent)
		pline = (dot (bcent - acent) dline) / length dline
		ponline = acent + (normalize dline * pline)
		
		--move line 1/3 towards B
		
		d = (bcent - ponline) / 3
		return #(acent + d, ccent + d)
	)	
	
	--==================================================================================================================================================
	-- aligntools use interface
	--==================================================================================================================================================
	button alignU "" pos:[8,11] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 6, 6, 6, 6) toolTip:"Align U"
	button alignv "" pos:[32,11] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 5, 5, 5, 5) toolTip:"Align V"
	button alignuv "" pos:[56,11] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 7, 7, 7, 7) toolTip:"Align UV"
	button straighten "" pos:[80,11] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 23, 23, 23, 23) toolTip:"Straighten"

	--==================================================================================================================================================
	-- aligntools events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on straighten pressed do 
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on straighten pressed do 
	(
		points = (uv.getselectedvertices()) as array
		
		if points.count < 3 do (messagebox "Need to select 3 or more points" title:"UvTools"; return undefined)
		
		v = #() -- list of verts
		vcount = 0
		av = [0,0,0]
		
		for i in points do
		(
			tvert = (uv.getvertexposition currenttime i)
			append v tvert
			--av += tvert
			vcount += 1
		)
		
		for i = 2 to vcount do
			av += (v[i] - v[1])
		av = [abs av.x, abs av.y, abs av.z]
		
		--choose majour axis of average points to sort by
		case of
		(
			((av.x > av.y) and (av.x > av.z)): (axis = #x)
			((av.y > av.z) and (av.y > av.x)): (axis = #y)
			((av.z > av.x) and (av.z > av.y)): (axis = #z)
		)
		
		bf = lineofbestfit v axis
		
		--project points onto best line
		bfline = bf[2] - bf[1]
		nbfline = normalize bfline
		lbfline = length bfline
	
		n = 0
		thehold.begin()
		for i in points do
		(
			d = dot bfline (v[n += 1] - bf[1]) / lbfline
			uv.setvertexposition2 currenttime i (bf[1] + (nbfline * d)) false false
		)
		
		thehold.accept "Straighten"
		uv.updateview()
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on alignu pressed do
	--
	--	Remarks:
	-- 		manages when the user wants to align vertically the vertices selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on alignu pressed do
	(
		local uvsel = uv.getselectedvertices() as array

		local mid = 0.0
		for i in uvsel do
			mid += (uv.getvertexposition currenttime i).x
		mid /= uvsel.count
		
		undo "Align U" on
		(
			uv.forceupdate false
			uv.movex mid
			uv.forceupdate true
			uv.updateview()
		)
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on alignv pressed do
	--
	--	Remarks:
	-- 		manages when the user wants to align horizontally the vertices selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on alignv pressed do
	(
		local uvsel = uv.getselectedvertices() as array

		local mid = 0.0
		for i in uvsel do
			mid += (uv.getvertexposition currenttime i).y
		mid /= uvsel.count
		
		undo "Align V" on
		(
			uv.forceupdate false
			uv.movey mid
			uv.forceupdate true
			uv.updateview()
		)
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on alignuv pressed do
	--
	--	Remarks:
	-- 		manages when the user wants to align vertically and horizontally at the same time the vertices selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on alignuv pressed do
	(
		local uvsel = uv.getselectedvertices() as array

		local mid = [0,0,0]
		for i in uvsel do
			mid += (uv.getvertexposition currenttime i)
		mid /= uvsel.count
		
		undo "Align UV" on
		(
			uv.forceupdate false
			uv.movex mid.x
			uv.movey mid.y
			uv.forceupdate true
			uv.updateview()
		)
	)		
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-









--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		Space Evenly rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout spacetools "Space Evenly" width:114 height:45
(
	--==================================================================================================================================================
	-- spacetools Variables
	--==================================================================================================================================================
	local uv = unwrapholder.uv

	--==================================================================================================================================================
	-- spacetools user interface
	--==================================================================================================================================================
	button spaceU "" pos:[21,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 24, 24, 24, 24) tooltip:"Space Evenly U"
	button spaceV "" pos:[45,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 25, 25, 25, 25) tooltip:"Space Evenly V"
	button spaceUV "" pos:[69,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 26, 26, 26, 26) tooltip:"Space Evenly UV"


	--==================================================================================================================================================
	-- spacetools functions
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function point3sort &p &indexes mode
	--
	--	Remarks:
	-- 		sorts the array p according to the component specified
	--	Parameters:
	--		p: array of vertices positions to sort
	--		indexes: indexes of the verts to sort
	--		mode: component against it will be sorted the array
	--	Return Value:
	--		none, all the arrays are modified inside the funtion
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function point3sort &p &indexes mode =
	(
 		if (mode == #x) do
		(
			for i = 1 to p.count - 1 do
				for n = (i + 1) to p.count do
					if (p[i].x > p[n].x) then
					(
						swap p[i] p[n]
						swap indexes[i] indexes[n]
					)
		)
	
		if (mode == #Y) do
		(
			for i = 1 to p.count - 1 do
				for n = (i + 1) to p.count do
					if p[i].y > p[n].y do
					(
						swap p[i] p[n]
						swap indexes[i] indexes[n]
					)
		)
	
		if (mode == #Z) do
		(
			for i = 1 to p.count - 1 do
				for n = (i + 1) to p.count do
					if (p[i].z > p[n].z) do
					(
						swap p[i] verts[n]
						swap indexes[i] indexes[n]
					)
		)		
	)	

	--==================================================================================================================================================
	-- spacetools events
	--==================================================================================================================================================
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on spaceU pressed do
	--
	--	Remarks:
	-- 		spaces the vertices selected evently horinzontaly
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on spaceU pressed do
	(
		tverts = #()
		tindex = #()
		count = 0

		for i in (uv.getselectedvertices()) do
		(
			append tverts (uv.getvertexposition currenttime i)
			append tindex i
			count += 1
		)
		
		point3sort &tverts &tindex #X
		
		width = tverts[count].x - tverts[1].x
		space = width / (count - 1)
		
		num = 0
		thehold.begin()
		for i = 1 to count  do
		(
			uv.setvertexposition2 currenttime tindex[i] [(tverts[1].x + (num * space)), tverts[i].y, tverts[i].z ] false false
			num += 1
		)
		uv.updateview()
		thehold.accept "Space U"
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on spaceV pressed do
	--
	--	Remarks:
	-- 		spaces the vertices selected evently vertically
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on spaceV pressed do
	(
		tverts = #()
		tindex = #()
		count = 0

		for i in (uv.getselectedvertices()) do

		(
			append tverts (uv.getvertexposition currenttime i)
			append tindex i
			count += 1
		)
		
		point3sort &tverts &tindex #Y
		
		width = tverts[count].y - tverts[1].y
		space = width / (count - 1)
		
		num = 0
		thehold.begin()
		for i = 1 to count do
		(
			uv.setvertexposition2 currenttime tindex[i] [tverts[i].x, (tverts[1].y + (num * space)), tverts[i].z ] false false
			num += 1
		)
		uv.updateview()
		thehold.accept "Space V"
	)	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on spaceUV pressed do
	--
	--	Remarks:
	-- 		spaces the vertices selected evently vertically and horiontally
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on spaceUV pressed do
	(
		tverts = #()
		tindex = #()
		count = 0

		for i in (uv.getselectedvertices()) do
		(
			append tverts (uv.getvertexposition currenttime i)
			append tindex i
			count += 1
		)
		
		point3sort &tverts &tindex #X
		
		width = tverts[count].x - tverts[1].x
		space = width / (count - 1)
		
		num = 0
		thehold.begin()
		for i = 1 to count do
		(
			uv.setvertexposition2 currenttime tindex[i] [(tverts[1].x + (num * space)), tverts[i].y, tverts[i].z ] false false
			tverts[i] = [(tverts[1].x + (num * space)), tverts[i].y, tverts[i].z ]
			num += 1
		)

		point3sort &tverts &tindex #Y
		
		width = tverts[count].y - tverts[1].y
		space = width / (count - 1)
		
		num = 0
		for i = 1 to count do
		(
			uv.setvertexposition2 currenttime tindex[i] [tverts[i].x, (tverts[1].y + (num * space)), tverts[i].z ] false false
			num += 1
		)			
		uv.updateview()
		thehold.accept "Space UV"
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-














--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		dumptools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout dumptools "Dump" width:114 height:149
(
	
	--==================================================================================================================================================
	-- dumptools Functions
	--==================================================================================================================================================
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function getuvtris &uvmod UVpoly = 
	(
		uvtrilist = #()
		
		for i in UVpoly do
		(
			if (np = uvmod.numberpointsinface i) > 3 then
			(
				for n = 1 to (np - 2) do
				(
					newuv = [0,0,0]
					newuv.x = uvmod.getvertexindexfromface i 1
					newuv.y = uvmod.getvertexindexfromface i (n + 1)
					newuv.z = uvmod.getvertexindexfromface i (n + 2)
					append uvtrilist newuv				
				)
			)
			else
			(


				newuv = [0,0,0]
				newuv.x = uvmod.getvertexindexfromface i 1
				newuv.y = uvmod.getvertexindexfromface i 2
				newuv.z = uvmod.getvertexindexfromface i 3
				append uvtrilist newuv
			)
		)
		return uvtrilist
	)	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function tridraw map p1 p2 p3 c1 c2 c3 
	--
	--	Remarks:
	-- 		draws a triangle in the bmp specified 
	--	Parameters:
	--		p1 p2 p3 are the points or the triangle
	--		c1 c2 c3 are the colors of the vertices of the triangle
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function tridraw map p1 p2 p3 c1 c2 c3 = 
	(
		--	sort points to top,mid,bot	
		v = #()
		colours = #()
		v[1] = [(p1.x * (map.width - 1)) as integer, (p1.y * (map.height - 1)) as integer, 0]
		v[2] = [(p2.x * (map.width - 1)) as integer, (p2.y * (map.height - 1)) as integer, 0]
		v[3] = [(p3.x * (map.width - 1)) as integer, (p3.y * (map.height - 1))as integer, 0]
		colours[1] = c1 as point3
		colours[2] = c2 as point3
		colours[3] = c3 as point3
		
		for i = 2 to 3 do
		(
			for n = 1 to 3 do
			(
				if v[i].y < v[n].y do
				(
					swap v[i] v[n]
					swap colours[i] colours[n]
				)
			)
		)
		
		--*	work out slopes	*
		slopeA = (v[1].x - v[2].x) / (v[1].y - v[2].y)
		slopeB = (v[3].x - v[1].x) / (v[3].y - v[1].y)
		slopeC = (v[2].x - v[3].x) / (v[2].y - v[3].y)
		
		--*	work out differences	*
		diffA = (v[2].y - v[1].y)
		diffB = (v[3].y - v[1].y)
		diffC = (v[3].y - v[2].y)
		
		cdiffA = (colours[2] - colours[1]) / diffA
		cdiffB = (colours[3] - colours[1]) / diffB
		cdiffC = (colours[3] - colours[2]) / diffC
		
		span1 = #() -- line a/b
		span2 = #() -- line a/c
		span3 = #() -- line b/c
		cspan1 = #()
		cspan2 = #()
		cspan3 = #()

		--*	get intersections for line B/A	*
		xs = v[1].x
		x = v[1].x
		y = v[1].y
			
		for i = 1 to diffA do
		(
			x = xs + (i * slopeA) as integer
			y += 1
			span1[i] = [x,y]
			cspan1[i] = (colours[1] + (i * cdiffA))
		)
	

		--*	get intersections for line A/C	*
		xs = v[1].x
		x = v[1].x
		y = v[1].y
	
		for i = 1 to diffB do
		(
			x = xs + (i * slopeB) as integer
			y += 1
			span2[i] = [x,y]
			cspan2[i] = (colours[1] + (i * cdiffB))
		)	
	
		--*	get intersections for line C/B	*
		xs = v[2].x
		x = v[2].x
		y = v[2].y
		
		for i = 1 to diffC do
		(
			x = xs + (i * slopeC) as integer
			y += 1
			span3[i] = [x,y]
			cspan3[i] = (colours[2] + (i * cdiffC))
		)
			
		--*	draw triangles	*
		draw1 = true
		tv = cross ((v[3] - v[1])) [0,0,1]
		if (dot tv ((v[2] - v[3]))) > 0.0 do draw1 = false
		
		if draw1 then
		(
			--*	draw top triangle	*
			for i = 1 to diffA do
			(
				col = #()
				d = (span2[i].x - span1[i].x + 1) -- how many pixels on span
				cmult = (cspan2[i] - cspan1[i]) / d
				for n = 1 to d do
				(
					col[n] = (cspan1[i] + (n * cmult)) as color
				)
				setpixels map span1[i] col
			)
			
			--*	draw bottom triangle	*
			for i = 1 to diffC do
			(
				col = #()
				d = (span2[i + diffA].x - span3[i].x + 1)
				cmult = (cspan2[i + diffA] - cspan3[i]) / d
				for n = 1 to d do
				(
					col[n] = (cspan3[i] + (n * cmult)) as color
				)
				setpixels map span3[i] col
			)			
		)
		else
		(
			--*	draw top triangle	*
			for i = 1 to diffA do
			(
				col = #()
				d = (span1[i].x - span2[i].x + 1)
				cmult = (cspan1[i] - cspan2[i]) / d
				for n = 1 to d do
				(
					col[n] = (cspan2[i] + (n * cmult)) as color
				)
				setpixels map span2[i] col
			)
	
			--*	draw bottom triangle	*
			for i = 1 to diffC do
			(
				col = #()
				d = (span3[i].x - span2[i + diffA].x + 1)
				cmult = (cspan3[i] - cspan2[i + diffA]) / d
				for n = 1 to d do
				(
					col[n] = (cspan2[i + diffA] + (n * cmult)) as color
				)
				setpixels map span2[i + diffA] col
			)		
		)
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function BLine bmp x1 y1 x2 y2 c 
	--
	--	Remarks:
	-- 		Bresenham line implementation for MAXScript
	-- 		by Harry Denholm, Kinetix 1998	
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------

	function BLine bmp x1 y1 x2 y2 c =
	(
		-- stick the colour into an array
		local ary = #(c)
		
		-- assign originals
		local Xb = x1 as integer
		local Yb = y1 as integer
		
		-- build the line deltas
		local dX = x2-x1 as float
		local dY = y2-y1 as float
		
		-- straight horiz 
		if dy == 0.0 do 
		(
			local xsign = if x2 < xb then -1 else 1
			setPixels bmp [xb,yb] ary
			while xb != x2 do
			(
				xb += xsign 
				setPixels bmp [xb,yb] ary
			)
			return true
		)
		-- straight vertical 
		if dx == 0.0 do 
		(	
			local ysign = if y2 < yb then -1 else 1
			setPixels bmp [xb,yb] ary
			while yb != y2 do
			(
				yb += ysign 
				setPixels bmp [xb,yb] ary
			)
			return true
		)
	
		-- no straights, go for bresenham line slide
		
		-- set up the movements
		local xsign = if x2 < xb then -1 else 1
		local ysign = if y2 < yb then -1 else 1
		dx = abs(dx)
		dy = abs(dy)
		setPixels bmp [xb,yb] ary
		
		-- line more vertical than horizontal
		if dx < dy then
		(
			local p = 2 * dx - dy
			local const1 = 2 * dx
			local const2 = 2 * (dx - dy)
			while yb != y2 do
			(
				yb += ysign
				if p < 0 then ( p += const1 )
				else
				(	
					p += const2
					xb += xsign
				)
				setPixels bmp [xb,yb] ary
			)
		)
		
		--	line more horizontal than vertical
		else
		(
			local p = 2 * dy - dx
			local const2 = 2 * (dy - dx)
			local const1 = 2 * dy
			while xb != x2 do
			(
				xb = xb + xsign
				if  p < 0 then ( p += const1 )
				else
				(	p += const2
					yb += ysign
				)
				setPixels bmp [xb,yb] ary
			)
		)
	)		
		
	
	--==================================================================================================================================================
	-- dumptools User Interface
	--==================================================================================================================================================
	
	dropDownList 	bitmappreset 	"" 			pos:[11,11]  width:92 height:21 items:#("128 X 128", "256 X 256", "512 X 512", "1024 X 1024")
	spinner 		bmpx 			"X:" 		pos:[21,36]  width:70 height:16 range:[1,3000,128] type:#integer
	spinner 		bmpy 			"Y:" 		pos:[21,55]  width:70 height:16 range:[1,3000,128] type:#integer
	button 			dodump 			"Dump Map" 	pos:[11,114] width:91 height:15
	progressBar 	pbar 			"" 			pos:[12,132] width:90 height:5 color:yellow
	radioButtons 	dumptype 		"" 			pos:[34,76]  width:46 height:32 labels:#("Wire", "Solid") columns:1

	--==================================================================================================================================================
	-- dumptools events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on bitmappreset selected val do
	--
	--	Remarks:
	-- 		resets the bitmap according to the size selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on bitmappreset selected val do
	(
		case val of
		(
			1: (bmpx.value = 128; bmpy.value = 128)
			2: (bmpx.value = 256; bmpy.value = 256)
			3: (bmpx.value = 512; bmpy.value = 512)
			4: (bmpx.value = 1024; bmpy.value = 1024)
		)
	) 
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dodump pressed do
	--
	--	Remarks:
	-- 		creates a new bitmap from unwrap modifier
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dodump pressed do
	(
		with redraw off
		(
			uv = modpanel.getcurrentobject()
			uv.forceupdate false
			bmp = bitmap bmpx.value bmpy.value
			sel = (uv.getselectedpolygons()) as array
			num = uv.numberpolygons()
			mode = #all
	
			if sel.count != 0 do
			(
				if (querybox "Dump Just selected?" title:"Unwrap") do
				(
					mode = #selected
					num = sel.count
				)
			)
			
			if mode == #all do
			(
				if dumptype.state == 1 then
				(
					for i = 1 to num do
					(
						inface = uv.numberpointsinface i
						startind = uv.getvertexindexfromface i 1
						startvert = uv.getvertexposition currenttime startind
						startvert = [startvert.x * (bmpx.value - 1), (1.0 - startvert.y) * (bmpy.value - 1), 0]
						prevvert = startvert
						prevind = startind
				
						for n = 2 to inface do
						(
							ind = uv.getvertexindexfromface i n
							newvert = uv.getvertexposition currenttime ind
							newvert = [newvert.x * (bmpx.value - 1), (1.0 - newvert.y) * (bmpy.value - 1), 0]
							BLine bmp prevvert.x (prevvert.y ) newvert.x (newvert.y) white
							prevvert = newvert
							prevind = ind
						)
				
						BLine bmp prevvert.x (prevvert.y ) startvert.x (startvert.y) white
						pbar.value = (i as float / num) * 100 
					)
				)
				else
				(
					uvtrilist = getuvtris &uv #{1..num}
					
					for i = 1 to num do
					(
						uv_a = uv.getvertexposition currenttime uvtrilist[i].x
						uv_b = uv.getvertexposition currenttime uvtrilist[i].y
						uv_c = uv.getvertexposition currenttime uvtrilist[i].z
						tridraw bmp [uv_a.x,1.0 - uv_a.y,uv_a.z] [uv_b.x,1.0 - uv_b.y,uv_b.z] [uv_c.x,1.0 - uv_c.y,uv_c.z] (color 255 0 0 1) (color 0 255 0 1) (color 0 0 255 1) 
						pbar.value = (i as float / num) * 100 
					)
				)
			)
	
			if mode == #selected do
			(
				if dumptype.state == 1 then
				(
					for i = 1 to num do
					(
						inface = uv.numberpointsinface sel[i]
						startind = uv.getvertexindexfromface sel[i] 1
						startvert = uv.getvertexposition currenttime startind
						startvert = [startvert.x * (bmpx.value - 1), (1.0 - startvert.y) * (bmpy.value - 1), 0]
						prevvert = startvert
						prevind = startind
				
						for n = 2 to inface do
						(
							ind = uv.getvertexindexfromface sel[i] n
							newvert = uv.getvertexposition currenttime ind
							newvert = [newvert.x * (bmpx.value - 1), (1.0 - newvert.y) * (bmpy.value - 1), 0]
							BLine bmp prevvert.x (prevvert.y) newvert.x (newvert.y) white
							prevvert = newvert
							prevind = ind
						)
				
						BLine bmp prevvert.x (prevvert.y) startvert.x (startvert.y) white
						pbar.value = (i as float / num) * 100 
					)
				)
				else
				(
					uvtrilist = getuvtris &uv sel
					
					ts = timestamp()
					for i = 1 to num do
					(
						uv_a = uv.getvertexposition currenttime uvtrilist[i].x
						uv_b = uv.getvertexposition currenttime uvtrilist[i].y
						uv_c = uv.getvertexposition currenttime uvtrilist[i].z
						tridraw bmp [uv_a.x,1.0 - uv_a.y,uv_a.z] [uv_b.x,1.0 - uv_b.y,uv_b.z] [uv_c.x,1.0 - uv_c.y,uv_c.z] (color 255 0 0 1) (color 0 255 0 1) (color 0 0 255 1) 
						pbar.value = (i as float / num) * 100 
					)
					format "time taken : % seconds\n" ((timestamp() - ts) / 1000.0)
				)
			)	
		)
		
		pbar.value = 0.0 
		display bmp
		uv.forceupdate false		
		uv.updateview()
	)	
)	
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-












--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		relaxtools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout relaxtools "Relax" width:114 height:122
(
	--==================================================================================================================================================
	-- relaxtools User Interface
	--==================================================================================================================================================

	spinner 	relitt 			"Iterations" 	pos:[17,12] width:79 height:16 range:[1,100,1] type:#integer
	spinner 	relstr 			"Strength" 		pos:[17,35] width:79 height:16 range:[0,1.0,0.1] type:#float
	checkbox 	rellockedge 	"Lock Edges" 	pos:[15,55] width:83 height:15
	checkbox 	relmatcharea 	"Match Area" 	pos:[15,74] width:83 height:15
	button 		dorelax 		"Relax" 		pos:[11,96] width:91 height:18
	button 		dorelax2 		"Relax 2" 		pos:[11,118] width:91 height:18
	button 		doFitRelax 		"Fit Relax" 	pos:[11,140] width:91 height:18
	
	--==================================================================================================================================================
	-- relaxtools events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dorelax pressed do
	--
	--	Remarks:
	-- 		relax the vertices selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dorelax pressed do
	(
		undo "Relax UV" on
		(
			uv = modpanel.getcurrentobject()
			uv.relax relitt.value relstr.value rellockedge.checked relmatcharea.checked
		)
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dorelax2 pressed do
	--
	--	Remarks:
	-- 		relax the vertices selected. Other method
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dorelax2 pressed do
	(
		undo "Relax2 UV" on
		(
			uv = modpanel.getcurrentobject()
			uv.relax2dialog()
		)
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on doFitRelax pressed do
	--
	--	Remarks:
	-- 		relax the vertices selected. Another method
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on doFitRelax pressed do
	(
		undo "Realx UV" on
		(
			uv = modpanel.getcurrentobject()
			uv.fitrelax relitt.value relstr.value
		)
	)	
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-












--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		indextools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout indextools "Index Rotate" width:114 height:43
(
	--==================================================================================================================================================
	-- indextools User Interface
	--==================================================================================================================================================

	button irotcw "" pos:[9,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 9, 9, 9, 9) toolTip:"index rotate clockwise"
	button indrotccw "" pos:[33,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 8, 8, 8, 8) toolTip:"index rotate counter clockwise"
	button quadmirrorX "" pos:[57,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 21, 21, 21, 21) toolTip:"quad mirror U"
	button quadmirrorY "" pos:[81,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 22, 22, 22, 22) toolTip:"quad mirror V"

	--==================================================================================================================================================
	-- indextools Events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on quadmirrorX pressed do
	--
	--	Remarks:
	-- 		quad mirror U
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on quadmirrorX pressed do
	(
		uv = modpanel.getcurrentobject()
		with redraw off
		(
			uv.forceupdate false
			if uv.gettvsubobjectmode() == 3 then -- only work on faces
			(
				sel = uv.getselectedfaces()
				for i in sel do
				(
					if uv.numberpointsinface i == 4 do
					(
						p1 = uv.getvertexindexfromface i 1
						p2 = uv.getvertexindexfromface i 2
						p3 = uv.getvertexindexfromface i 3
						p4 = uv.getvertexindexfromface i 4
						
						uv.setfacevertexindex i 1 p2
						uv.setfacevertexindex i 2 p1
						uv.setfacevertexindex i 3 p4

						uv.setfacevertexindex i 4 p3						
					)
				)
			)

			else
			(
				messagebox "This only works in face mode on quad polygons"  title:"UvTools"
			)
		)
		uv.forceupdate true
		uv.updateview()		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on quadmirrorY pressed do
	--
	--	Remarks:
	-- 		quad mirror V
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on quadmirrorY pressed do
	(
		uv = modpanel.getcurrentobject()
		with redraw off
		(
			uv.forceupdate false
			if uv.gettvsubobjectmode() == 3 then -- only work on faces
			(
				sel = uv.getselectedfaces()
				for i in sel do
				(
					if uv.numberpointsinface i == 4 do
					(
						p1 = uv.getvertexindexfromface i 1
						p2 = uv.getvertexindexfromface i 2
						p3 = uv.getvertexindexfromface i 3
						p4 = uv.getvertexindexfromface i 4
	
						uv.setfacevertexindex i 1 p4
						uv.setfacevertexindex i 2 p3
						uv.setfacevertexindex i 3 p2
						uv.setfacevertexindex i 4 p1						
					)
				)
			)
			else
			(
				messagebox "This only works in face mode on quad polygons"  title:"UvTools"
			)
		)
		uv.forceupdate true
		uv.updateview()		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on irotccw pressed do
	--
	--	Remarks:
	-- 		index rotate counter clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on irotccw pressed do
	(
		uv = modpanel.getcurrentobject()
		
		sel = uv.getselectedpolygons()
		uv.selectpolygonsupdate sel false
		thehold.begin()
		for i in sel do
		(
			inface = uv.numberpointsinface i
			index = #()
	
			for n = 1 to inface do
				append index (uv.getvertexindexfromface i n)

			index = #(index[inface]) + index
			deleteitem index (inface + 1)
			
			for n = 1 to inface do
				uv.setfacevertexindex i n index[n]

		)
		thehold.accept "Index Rotate CCW"
		uv.selectpolygonsupdate sel true
		uv.forceupdate true
		uv.updateview()
				
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on irotcw pressed do
	--
	--	Remarks:
	-- 		index rotate clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on irotcw pressed do
	(
		uv = modpanel.getcurrentobject()
		
		sel = uv.getselectedpolygons()
		uv.selectpolygonsupdate sel false
		
		for i in sel do
		(
			inface = uv.numberpointsinface i
			index = #()
	
			for n = 1 to inface do
				append index (uv.getvertexindexfromface i n)

			append index index[1]
			deleteitem index 1
			
			for n = 1 to inface do
				uv.setfacevertexindex i n index[n]
		)
		uv.selectpolygonsupdate sel true
		uv.forceupdate true
		uv.updateview()		
	)		
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-












--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		softSelTools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout softSelTools "Soft Selection" width:114 height:45
(
--	checkbox dash_SoftSel "Use SoftSelection" pos:[5,8] width:103 height:16 checked:false
	radiobuttons dash_UVSpace "" pos:[16,6] width:83 height:16 enabled:true labels:#("XY", "UV") columns:2
	spinner dash_SoftFalloff "Falloff :" pos:[16,35] width:90 height:16 enabled:false range:[0,100,0.2] scale:0.01 fieldwidth:40
 	checkbutton dash_FalloffSin "" pos:[3,56] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn,  47, 52, 47, 52)   toolTip:"Smooth Falloff"
    checkbutton dash_FalloffLin "" pos:[31,56] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 48, 53, 48, 53)   toolTip:"Linear Falloff"
	checkbutton dash_FalloffIn "" pos:[58,56] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 49, 54, 49, 54)   toolTip:"Slow Out Falloff"
	checkbutton dash_FalloffOut "" pos:[85,56] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 50, 55, 50, 55)   toolTip:"Fast Out Falloff"
	spinner dash_EdgeDistAmount "Edge dist." pos:[28,91] width:78 height:16 enabled:true range:[0,50,4] type:#integer scale:0.1 fieldwidth:25
	checkbox dash_edgeDist "" pos:[5,91] width:18 height:16 checked:false
	
	groupBox grp4 "" pos:[-6,21] width:140 height:93
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		fn UpdateUI 
	--
	--	Remarks:
	-- 		updates the user interface of the rollout
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	fn UpdateUI = 
	(
		unwrapMod = unwrapholder.uv
	
		-- Get all selection Modes and set the UI correctly
		--dash_SoftSel.checked = unwrapMod.getEnableSoftSelection() 

		dash_SoftFalloff.value = unwrapMod.getFalloffDist() 
		if 	unwrapMod.getEnableSoftSelection() == true do dash_SoftFalloff.enabled = true
		if 	unwrapMod.getEnableSoftSelection() == false do dash_SoftFalloff.enabled = false

		dash_FalloffLin.state = false 
		dash_FalloffSin.state = false
		dash_FalloffIn.state = false
		dash_FalloffOut.state = false
		dash_UVSpace.state = unwrapMod.getFalloffSpace()
		
		if unwrapMod.getFalloffType() == 1 do dash_FalloffLin.state = true 
		if unwrapMod.getFalloffType() == 2 do dash_FalloffSin.state = true
		if unwrapMod.getFalloffType() == 3 do dash_FalloffIn.state = true
		if unwrapMod.getFalloffType() == 4 do dash_FalloffOut.state = true
		dash_EdgeDistAmount.value  = unwrapMod.getLimitSoftSelRange()
		dash_edgeDist.state =  unwrapMod.getLimitSoftSel()
	)
	
-- Soft Selection UI items

/*	on dash_SoftSel changed state do
	(
		unwrapMod = modpanel.getcurrentobject()
		if state == false do 
			(
--			unwrapMod.SetFalloffDist 0
			unwrapMod.setEnableSoftSelection false
			dash_SoftFalloff.enabled = false
			)
		if state == true do
			(
--			unwrapMod.SetFalloffDist dash_SoftFalloff.value
			unwrapMod.setEnableSoftSelection true
			dash_SoftFalloff.enabled = true
			)
	)*/
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_FalloffLin changed state do 
	--
	--	Remarks:
	-- 		Sets the falloff type in the edit floater, where falloff defines the falloff type
	--			1 - Smooth falloff, 2 - Linear falloff, 3 - Slow Out falloff, 4 - Fast Out falloff, 
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_FalloffLin changed state do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setFalloffType 1
		dash_FalloffLin.checked = true
		dash_FalloffSin.checked = false
		dash_FalloffIn.checked = false
		dash_FalloffOut.checked = false

	)
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_FalloffSin changed state do 
	--
	--	Remarks:
	-- 		Sets the falloff type in the edit floater, where falloff defines the falloff type
	--			1 - Smooth falloff, 2 - Linear falloff, 3 - Slow Out falloff, 4 - Fast Out falloff, 
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_FalloffSin changed state do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setFalloffType 2
		dash_FalloffLin.checked = false
		dash_FalloffSin.checked = true
		dash_FalloffIn.checked = false
		dash_FalloffOut.checked = false
		
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_FalloffIn changed state do 
	--
	--	Remarks:
	-- 		Sets the falloff type in the edit floater, where falloff defines the falloff type
	--			1 - Smooth falloff, 2 - Linear falloff, 3 - Slow Out falloff, 4 - Fast Out falloff, 
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_FalloffIn changed state do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setFalloffType 3
		dash_FalloffLin.checked = false
		dash_FalloffSin.checked = false
		dash_FalloffIn.checked = true
		dash_FalloffOut.checked = false
		
	)
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_FalloffOut changed state do
	--
	--	Remarks:
	-- 		Sets the falloff type in the edit floater, where falloff defines the falloff type
	--			1 - Smooth falloff, 2 - Linear falloff, 3 - Slow Out falloff, 4 - Fast Out falloff, 
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_FalloffOut changed state do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setFalloffType 4
		dash_FalloffLin.checked = false
		dash_FalloffSin.checked = false
		dash_FalloffIn.checked = false
		dash_FalloffOut.checked = true		
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_SoftFalloff changed val do
	--
	--	Remarks:
	-- 		Sets the falloff distance in the edit floater
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_SoftFalloff changed val do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.SetFalloffDist val
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_UVSpace changed stat do 
	--
	--	Remarks:
	-- 		Sets the falloff space in the edit floater, where space defines the space you want the falloff to be computed in
	--			1 - XY (The local space of the object.), 2 - UV (The UVW space of the object.)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_UVSpace changed stat do 
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.SetFalloffSpace stat
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_EdgeDistAmount changed val do
	--
	--	Remarks:
	-- 		Sets the Edge Distance value used to limit the Soft Selection
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_EdgeDistAmount changed val do 
	(
   		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setLimitSoftSelRange val
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dash_edgeDist changed state do
	--
	--	Remarks:
	-- 		When the state is true as limit, the Edge Distance will be used
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dash_edgeDist changed state do
	(
		unwrapMod = modpanel.getcurrentobject()
		unwrapMod.setLimitSoftSel state 
	)
	
	on softSelTools open do
		UpdateUI()
		
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-









--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
--
--	Rollout:
--		centretools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
rollout centretools "Centre Point Snap" width:114 height:96
(
	
	--==================================================================================================================================================
	-- centretools User Interface
	--==================================================================================================================================================
	
	button csnap1 "" pos:[21,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 12, 12, 12, 12) tootip:"Top Left"
	button csnap2 "" pos:[45,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 13, 13, 13, 13) tootip:"Top Middle"
	button csnap3 "" pos:[69,10] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 14, 14, 14, 14) tootip:"Top Right"
	button csnap4 "" pos:[21,35] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 11, 11, 11, 11) tootip:"Left"
	button csnap5 "" pos:[45,35] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 18, 18, 18, 18) tootip:"Middle"
	button csnap6 "" pos:[69,35] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 15, 15, 15, 15) tootip:"Right"
	button csnap7 "" pos:[21,60] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 10, 10, 10, 10) tootip:"Bottom Left"
	button csnap8 "" pos:[45,60] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 17, 17, 17, 17) tootip:"Bottom Middle"
	button csnap9 "" pos:[69,60] width:24 height:24 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 16, 16, 16, 16) tootip:"Bottom Right"

	--==================================================================================================================================================
	-- centretools events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap1 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( upper left of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap1 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 8		

	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap2 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( upper center of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap2 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 7		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap3 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( upper right of the selection ) 
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap3 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 6		
	)		
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap4 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( left center of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap4 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 9		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap5 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. (  center )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap5 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 1		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap6 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( right center of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap6 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 5		
	)	

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap7 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( upper center of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap7 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 2		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap8 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( lower center of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap8 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 3		
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on csnap9 pressed do
	--
	--	Remarks:
	-- 		This lets you quickly snap the free form gizmo pivot to the bounding rectangle. ( lower right of the selection )
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on csnap9 pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.snappivot 4		
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-





--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		transformtools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout transformtools "Transform Tools" width:114 height:318
(
	--==================================================================================================================================================
	-- transformtools Variables
	--==================================================================================================================================================
	struct dims (width, height, mid, maxUV, minUV)
	local uv = unwrapholder.uv
	
	--==================================================================================================================================================
	-- transformtools User Interface
	--==================================================================================================================================================

	spinner uoffs 	"U:" 		pos:[24,10] width:74 height:16 range:[-99,992,0] scale:0.0001
	spinner voffs 	"V:" 		pos:[24,29] width:74 height:16 range:[-99,99,0] scale:0.0001
	
	button dooffs 	"Offset" 	pos:[5,52] width:50 height:16
	button doscale 	"Scale" 	pos:[57,52] width:50 height:16

	button doExpand "Expand" 	pos:[5,72] width:50 height:16
	button doCompact "Compact" 	pos:[57,72] width:50 height:16
		
	button rot45cw 		"" pos:[16,98] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 38, 38, 38, 38) tooltip:"Rotate -45"
	button rot90cw 		"" pos:[43,98] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 37, 37, 37, 37) tooltip:"Rotate -90"
	button rot180cw 	"" pos:[70,98] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 36, 36, 36, 36) tooltip:"Rotate -180"
	button rot45ccw 	"" pos:[16,130] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 39, 39, 39, 39) tooltip:"Rotate 45"
	button rot90ccw 	"" pos:[43,130] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 40, 40, 40, 40) tooltip:"Rotate 90"
	button rot180ccw 	"" pos:[70,130] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 41, 41, 41, 41) tooltip:"Rotate 180"

	button flipU 	"" pos:[30,162] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 21, 21, 21, 21) tooltip:"Mirror U"
	button flipV 	"" pos:[57,162] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 22, 22, 22, 22) tooltip:"Mirror V"

	button nup 		"" pos:[43,198] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 44, 44, 44, 44) toolTip:"Nudge Up"
	button ndown 	"" pos:[43,227] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 45, 45, 45, 45) toolTip:"Nudge Down"
	button nleft 	"" pos:[14,214] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 43, 43, 43, 43) toolTip:"Nudge Left"
	button nright 	"" pos:[72,214] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 42, 42, 42, 42) toolTip:"Nudge Right"

	radioButtons 	nudgemode "Nudge Mode:" pos:[5,260] width:121 height:30 labels:#("Units", "Pixels") columns:2
	spinner 		nudgeamount "Amount:" pos:[14,295] width:95 height:16 type:#float
	
	
	--==================================================================================================================================================
	-- transformtools functions
	--==================================================================================================================================================
	
	--==================================================================================================================================================
	-- transformtools events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on nudegmode changed state do
	--
	--	Remarks:
	-- 		allows the user to specify pixels or units when he uses the nudge options
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on nudegmode changed state do
	(
		if state == 1 then
			nudegamount.type = #float
		else
			nudegamount.type = #integer
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on nup pressed do
	--
	--	Remarks:
	-- 		Nudge Up the selected vertices
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on nup pressed do
	(
		if nudgemode.state == 1 then
		(
			offset = nudgeamount.value
		)
		else
		(
			offset = nudgeamount.value * (1.0 / uv.getrenderheight())
			if offset == 0 do offset = nudgeamount.value * 0.00390625
		)
			
		undo "Nudge Up" on
		(		
			uv.moveselectedvertices [0,offset,0]
		)
	)
		
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on ndown pressed do
	--
	--	Remarks:
	-- 		Nudge Down the selected vertices
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on ndown pressed do
	(
		if nudgemode.state == 1 then
		(
			offset = nudgeamount.value
		)
		else
		(
			offset = nudgeamount.value * (1.0 / uv.getrenderheight())
			if offset == 0 do offset = nudgeamount.value * 0.00390625
		)
			
		undo "Nudge Down" on
		(		
			uv.moveselectedvertices [0,-offset,0]
		)
	)	
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on nleft pressed do
	--
	--	Remarks:
	-- 		Nudge Left the selected vertices
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on nleft pressed do
	(
		if nudgemode.state == 1 then
		(
			offset = nudgeamount.value
		)
		else
		(
			offset = nudgeamount.value * (1.0 / uv.getrenderwidth())
			if offset == 0 do offset = nudgeamount.value * 0.00390625
		)
			
		undo "Nudge Left" on
		(
			uv.moveselectedvertices [-offset,0,0]
		)
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on nright pressed do
	--
	--	Remarks:
	-- 		Nudge Right the selected vertices
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on nright pressed do
	(
		if nudgemode.state == 1 then
		(
			offset = nudgeamount.value
		)
		else
		(
			offset = nudgeamount.value * (1.0 / uv.getrenderwidth())
			if offset == 0 do offset = nudgeamount.value * 0.00390625
		)
	
		undo "Nudge Right" on
		(		
			uv.moveselectedvertices [offset,0,0]
		)
	)	
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on flipU pressed do
	--
	--	Remarks:
	-- 		mirrors the verts selected horizontally
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on flipU pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.mirrorh()
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on flipV pressed do
	--
	--	Remarks:
	-- 		mirrors the verts selected vertically
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on flipV pressed do
	(
		uv = modpanel.getcurrentobject()
		uv.mirrorv()
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot45cw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 45 grades clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot45cw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter -0.785398
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot90cw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 90 grades clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot90cw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter -1.5708
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot180cw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 180 grades clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot180cw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter -3.14159
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot45ccw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 45 grades counter clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot45ccw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter 0.785398
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot90ccw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 90 grades counter clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot90ccw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter 1.5708
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on rot180ccw pressed do
	--
	--	Remarks:
	-- 		Rotates the selected vertices around their center point 180 grades counter clockwise
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on rot180ccw pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		uv.RotateSelectedVerticesCenter 3.14159
	)		
		
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on doscale pressed do
	--
	--	Remarks:
	-- 		scales the selected vertices by the amount specified in the UV spinners of the rollout
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on doscale pressed do
	(
		errorstring = ""
		errormode = false
		
		if uoffs.value == 0.0 do (errormode = true; errorstring += " U Offset ")
		if voffs.value == 0.0 do (errormode = true; errorstring += " V Offset ")
		
		if errormode do
		(
			if not (querybox ("Your " + errorstring + " is set to 0.0\nAre you sure?") title:"Confirm action") do
			(
				return false
			)
		)
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv

		undo "UV Scale" on
		(
			uv.scaleselectedverticescenter uoffs.value 1
			uv.scaleselectedverticescenter voffs.value 2
		)

		uv.updateview()
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on dooffs pressed do
	--
	--	Remarks:
	-- 		offsets the selected vertices by the amount specified in the UV spinners of the rollout
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on dooffs pressed do
	(
		uv = modpanel.getcurrentobject()

		unwrapholder.seltovertmode uv
		undo "UV Offset" on
		(
			uv.moveselectedvertices [uoffs.value, voffs.value, 0]
		)
		uv.updateview()
	)
	
	fn getMatIdList =
	(
		local obj = selection[1]
		local matIds = #{}
		
		local getfaceMatIdFn = if (classof obj == PolyMeshObject) then polyop.getfacematid else getfacematid 
		for f in obj.faces do 
			append matIds (getfaceMatIdFn obj f.index)
					
		return matIds
	)
	
	fn compactSelection =
	(
		local sel = uv.getselectedvertices()

		for i in sel do
		(
			tv = (uv.getvertexposition currenttime i)
			tv.x -= ceil(tv.x) - 1.0
			uv.setvertexposition2 currenttime i tv false false
		)
		uv.updateview()
	)	
	
	fn doCompactFn =
	(
		local matIds = getMatIdList()
		for id in matIds do
		(
			uv.selectByMatID id
			uv.GetSelectionFromFace()
			compactSelection()
		)		
	)
	
	on doCompact pressed do
	(
		thehold.begin()
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		undo "UV Expand" on
		(
			doCompactFn()
		)
		uv.updateview()
		thehold.accept "UV Compact"
	)
	
	on doExpand pressed do
	(
		doCompactFn()
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		undo "UV Expand" on
		(
			local matIds = getMatIdList()
	
			local offset = -1
			for id in matIds do
			(
				uv.selectByMatID id
				uv.GetSelectionFromFace()
				uv.moveselectedvertices [offset+=1, 0, 0]
			)
				
		)
		uv.updateview()
	)
	

)	
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-






--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		tiletools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout tiletools "Tile Tools" width:114 height:177
(
	--==================================================================================================================================================
	-- tiletools Variables
	--==================================================================================================================================================
	struct dims (width, height, mid, maxUV, minUV)
	
	--==================================================================================================================================================
	-- tiletools User Interface
	--==================================================================================================================================================
	button seldim 			"Dimensions" 		pos:[11,13] width:91 height:18
	button centersel 		"Center" 			pos:[51,34] width:51 height:45
	button maxtile 			"Max Tiling Range" 	pos:[11,84] width:91 height:18
	spinner xtile 			"X" 				pos:[25,107] width:63 height:16 range:[0.1,100,1]
	spinner ytile 			"Y" 				pos:[25,128] width:63 height:16 range:[0.1,100,1]
	radioButtons centertype "" 					pos:[10,33] width:38 height:48 labels:#("U", "V", "UV") columns:1
	checkbox aspect 		"Keep Aspect?" 		pos:[11,150] width:91 height:14

	--==================================================================================================================================================
	-- tiletools events
	--==================================================================================================================================================
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on centersel pressed do
	--
	--	Remarks:
	-- 		center the vertex selected
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on centersel pressed do 
	(
		uv = modpanel.getcurrentobject()
		sel = uv.getselectedvertices()
		
		d = unwrapholder.getDims sel &uv

		offset = [0.5,0.5,0] - d.mid
		
		case centertype.state of
		(
			1: offset.y = 0.0
			2: offset.x = 0.0
		)

		for i in sel do
		(
			p = uv.getvertexposition currenttime i
			uv.setvertexposition2 currenttime i (p + offset) false false
		)
		uv.updateview()
	)		
		
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on seldim pressed do 
	--
	--	Remarks:
	-- 		shows the dimension of the bounding box containing all the selected verts
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on seldim pressed do 
	(
		uv = modpanel.getcurrentobject()
		sel = uv.getselectedvertices()
		
		d = unwrapholder.getDims sel &uv
	
		Messagebox ("width = " + d.width as string + "\nHeight = " + d.height as string + "\nCentre = " + d.mid as string + "\nMax UV = " + d.maxUV as string + "\nMin UV = " + d.minUV as string) title:"Dimensions"
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on maxtile pressed do
	--
	--	Remarks:
	-- 		fits the selected verts in the range specified in the UI
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on maxtile pressed do 
	(
		uv = modpanel.getcurrentobject()
		sel = uv.getselectedvertices()
		
		d = unwrapholder.getDims sel &uv
		
		-- scale if needed
		scX = 1.0
		scY = 1.0
		offset = d.mid
		
		if d.width > xtile.value do (scX = xtile.value / d.width; offset.x = 0.5)
		if d.height > ytile.value do (scY = ytile.value / d.height; offset.y = 0.5)
		
		if (scX == 1.0) and (scY == 1.0) do return false-- exit if no scale
		
		--fit biggest to keep aspect
		amtX = abs (1.0 - scX)

		amtY = abs (1.0 - scY)
		
		if aspect.checked do	
		(
			if amtX > amtY then
			(
				scY = scX
			)
			else
			(
				if amtY > amtX do scX = scY
			)
		)
		
		for i in sel do
		(
			p = (uv.getvertexposition currenttime i) - d.mid
			uv.setvertexposition2 currenttime i ([p.x * scX ,p.y * scY ,0.0] + offset) false false
		)
		uv.updateview()
	)	
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-











--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		moretools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout moretools "More Tools" width:114 height:90
(
	--==================================================================================================================================================
	-- moretools User Interface
	--==================================================================================================================================================

	button unusedasdead "Clear Unused" 		pos:[11,13] width:91 height:15
	progressBar pb2 	"" 					pos:[12,34] width:90 height:5 color:yellow
	button selopenedge 	"Sel Open Edges" 	pos:[11,44] width:91 height:15
	button fullmap 		"0 to 1 Fit" 		pos:[11,63] width:91 height:15

	--==================================================================================================================================================
	-- moretools events
	--==================================================================================================================================================

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on unusedasdead pressed do
	--
	--	Remarks:
	-- 		Marks the non used vertices that it is dead, no longer in use
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on unusedasdead pressed do
	(
		with redraw off
		(
			uv = modpanel.getcurrentobject()
			uv.forceupdate false
			numpoly = uv.numberpolygons()
			numverts = uv.numbervertices()
			
			used = #{}
		
			for i = 1 to numpoly do
			(
				for n = 1 to (uv.numberpointsinface i) do
					used[(uv.getvertexindexfromface i n)] = true

				pb2.value = (i as float / numpoly) * 100
			)
	
			unused = #{1..numverts} - used

			for i in unused do
				uv.markasdead i
		
			pb2.value = 0.0
			uv.forceupdate true
			uv.updateview()
		)
	
	)
		
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on fullmap pressed do
	--
	--	Remarks:
	-- 		scale all the selected verts to fit in the 0 to 1 map
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on fullmap pressed do
	(
		uv = modpanel.getcurrentobject()
		unwrapholder.seltovertmode uv
		sel = uv.getselectedvertices()
		d = unwrapholder.getDims sel &uv
		
		scw = 1.0 / d.width
		sch = 1.0 / d.height
		
		if (querybox "Keep Aspect?" title:"0 to 1 map") do
		(
			if d.width > d.height then
			(
				sch = scw
			)
			else
			(
				scw = sch
			)
		)

		thehold.begin()
		for i in sel do
		(
			tv = (uv.getvertexposition currenttime i) - d.mid
			uv.setvertexposition2 currenttime i ((tv * [scw,sch,0]) + [0.5,0.5,0.0]) false false
		)
		uv.updateview()
		thehold.accept "0 to 1 Map"
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on selopenedge pressed do
	--
	--	Remarks:
	-- 		selects the selected edges
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on selopenedge pressed do
	(
		uv = modpanel.getcurrentobject()
		
				
		case uv.gettvsubobjectmode() of
		(
			1: (uv.verttoedgeselect())
			3: (uv.facetoedgeselect())
		)
		
		sel = (uv.getselectededges()) as array
		print sel.count
		
		if sel.count == 0 do
		(
			messagebox "Please make a selection first!" title:"UvTools"
			return false
		)
			
		oldsel = uv.getselectededges()
		uv.openedgeselect()
		newsel = (uv.getselectededges()) - oldsel
		uv.selectedges newsel
		uv.openedgeselect()
		uv.settvsubobjectmode 2
		unwrapholder.subObjVert.state = false
		unwrapholder.subObjEdge.state = true
		unwrapholder.subObjFace.state = false
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
















--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		deformtools rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout deformtools "Deform Tools" width:114 height:257
(	
	--==================================================================================================================================================
	-- deformtools Variables
	--==================================================================================================================================================
	struct dims (width, height, mid, maxUV, minUV)
	local UVs = #()
	local uv, sel, dim, offset, range
	local activecentre = 5
	
	--==================================================================================================================================================
	-- deformtools Functions
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function setcentre num
	--
	--	Remarks:
	-- 		sets the pivot point from wich all the deform operations will be based
	--	Parameters:
	--		num: indicates the position of the pivot point. from left top to right bottom, 	1 2 3  
	--																						4 5 6
	--																						7 8 9
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function setcentre num =
	(
		deformtools.activecentre = num
		bits = #{1..9}
		bits[num] = false
		
		deformtools.cent1.checked = not bits[1]
		deformtools.cent2.checked = not bits[2]
		deformtools.cent3.checked = not bits[3]
		deformtools.cent4.checked = not bits[4]
		deformtools.cent5.checked = not bits[5]
		deformtools.cent6.checked = not bits[6]
		deformtools.cent7.checked = not bits[7]
		deformtools.cent8.checked = not bits[8]
		deformtools.cent9.checked = not bits[9]
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function getsetup
	--
	--	Remarks:
	-- 		
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function getsetup =
	(
		deformtools.uvs = #()
		deformtools.uv = modpanel.getcurrentobject()
		deformtools.sel = (deformtools.uv.getselectedvertices()) as array
		deformtools.dim = unwrapholder.getDims sel &uv
				
		case deformtools.activecentre of
		(
			1: deformtools.offset = [deformtools.dim.minuv.x, deformtools.dim.maxuv.y, 0]
			2: deformtools.offset = [deformtools.dim.mid.x, deformtools.dim.maxuv.y, 0]
			3: deformtools.offset = [deformtools.dim.maxuv.x, deformtools.dim.maxuv.y, 0]
			4: deformtools.offset = [deformtools.dim.minuv.x, deformtools.dim.mid.y, 0]
			5: deformtools.offset = [deformtools.dim.mid.x, deformtools.dim.mid.y, 0]
			6: deformtools.offset = [deformtools.dim.maxuv.x, deformtools.dim.mid.y, 0]
			7: deformtools.offset = [deformtools.dim.minuv.x, deformtools.dim.minuv.y, 0]
			8: deformtools.offset = [deformtools.dim.mid.x, deformtools.dim.minuv.y, 0]
			9: deformtools.offset = [deformtools.dim.maxuv.x, deformtools.dim.minuv.y, 0]
		)
		
		
		for i in sel do
		(
			append deformtools.uvs ((deformtools.uv.getvertexposition currenttime i) - deformtools.offset)
		)	
		
		deformtools.dim.maxuv -= deformtools.offset
		deformtools.dim.minuv -= deformtools.offset
		deformtools.range = deformtools.dim.maxuv - deformtools.dim.minuv
	)
	
	--==================================================================================================================================================
	-- deformtools user interface
	--==================================================================================================================================================
	
	spinner skewv 	"Skew V" pos:[14,130] width:92 height:16 range:[-90,90,0] scale:0.01
	spinner skewU 	"Skew U" pos:[14,110] width:92 height:16 range:[-90,90,0] scale:0.01
	timer updatetimer_skewu interval:30 active:false
	timer updatetimer_skewv interval:30 active:false	
	
	spinner taperv 	"Taper V" pos:[14,179] width:92 height:16 range:[-90,90,0] scale:0.01
	spinner taperu 	"Taper U" pos:[14,159] width:92 height:16 range:[-90,90,0] scale:0.01
	timer updatetimer_taperu interval:30 active:false
	timer updatetimer_taperv interval:30 active:false


	spinner bendv 	"Bend V" pos:[14,231] width:92 height:16 range:[-90,90,0] scale:0.01 enabled:false
	spinner bendu 	"Bend U" pos:[14,211] width:92 height:16 range:[-90,90,0] scale:0.01 enabled:false
	timer updatetimer_bendu interval:30 active:false
	timer updatetimer_bendv interval:30 active:false

	checkbutton cent1 "" pos:[16,21] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 12, 3, 12, 3)
	checkbutton cent2 "" pos:[43,21] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 13, 4, 13, 4)
	checkbutton cent3 "" pos:[70,21] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 14, 19, 14, 19)

	checkbutton cent4 "" pos:[16,48] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 11, 2, 11, 2)
	checkbutton cent5 "" pos:[43,48] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 18, 35, 18, 35) checked:true
	checkbutton cent6 "" pos:[70,48] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 15, 20, 15, 20)

	checkbutton cent7 "" pos:[16,75] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 10, 1, 10, 1)
	checkbutton cent8 "" pos:[43,75] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 17, 34, 17, 34)
	checkbutton cent9 "" pos:[70,75] width:27 height:27 images:#(unwrapholder.uviconsI, undefined, unwrapholder.icn, 16, 33, 16, 33)
	label lbl1 "Centre Point" pos:[27,5] width:59 height:13

	--==================================================================================================================================================
	-- centre points events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		cent1 to cent9 
	--
	--	Remarks:
	-- 		sets the pivot point
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on cent1 changed arg do
		setcentre 1

	on cent2 changed arg do
		setcentre 2

	on cent3 changed arg do
		setcentre 3

	on cent4 changed arg do
		setcentre 4

	on cent5 changed arg do
		setcentre 5

	on cent6 changed arg do
		setcentre 6

	on cent7 changed arg do
		setcentre 7

	on cent8 changed arg do
		setcentre 8

	on cent9 changed arg do
		setcentre 9

	--==================================================================================================================================================
	-- skews events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on skewU buttondown do
	--
	--	Remarks:
	-- 		skew vertically the selected vertices (buttondown event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on skewU buttondown do
	(
		getsetup()
		updatetimer_skewu.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on updatetimer_skewu tick do
	--
	--	Remarks:
	-- 		used to control the skewU action
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on updatetimer_skewu tick do
	(
		updatetimer_skewu.active = false
		with redraw off
		(
			for i = 1 to sel.count do
			(
				amt = (uvs[i].x / range.x) * skewu.value
				uv.setvertexposition2 currenttime sel[i] ([uvs[i].x, uvs[i].y + amt, uvs[i].z] + offset) false false
			)
		)
		
		uv.updateview()	
		updatetimer_skewu.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on skewu buttonup do
	--
	--	Remarks:
	-- 		skew vertically the selected vertices (buttonup event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on skewu buttonup do
	(
		updatetimer_skewu.active = false
		skewu.value = 0.0
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on skewV buttondown do
	--
	--	Remarks:
	-- 		skew horizontally the selected vertices (buttondown event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on skewV buttondown do
	(
		getsetup()
		updatetimer_skewv.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on updatetimer_skewv tick do
	--
	--	Remarks:
	-- 		used to control the skewV action
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on updatetimer_skewv tick do
	(
		updatetimer_skewv.active = false
		with redraw off
		(
			for i = 1 to sel.count do
			(
				amt = (uvs[i].y / range.y) * skewv.value
				uv.setvertexposition2 currenttime sel[i] ([uvs[i].x + amt, uvs[i].y, uvs[i].z] + offset) false false
			)
		)
		uv.updateview()	
		updatetimer_skewv.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on skewV buttonup do
	--
	--	Remarks:
	-- 		skew horizontally the selected vertices (buttonup event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on skewV buttonup do
	(
		updatetimer_skewv.active = false
		skewv.value = 0.0
	)

	--==================================================================================================================================================
	-- taper events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on taperU buttondown do
	--
	--	Remarks:
	-- 		tape vertically the selected vertices (buttondown event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on taperU buttondown do
	(
		getsetup()
		updatetimer_taperu.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on updatetimer_taperu tick do
	--
	--	Remarks:
	-- 		used to control the taperU action
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on updatetimer_taperu tick do
	(
		updatetimer_taperu.active = false
		with redraw off
		(
			for i = 1 to sel.count do
			(
				amt = (uvs[i].x / range.x) * taperu.value
				uv.setvertexposition2 currenttime sel[i] ([uvs[i].x, uvs[i].y * (1.0 + amt), uvs[i].z] + offset) false false
			)
		)
		uv.updateview()	
		updatetimer_taperu.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on taperu buttonup do
	--
	--	Remarks:
	-- 		tape vertically the selected vertices (buttonup event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on taperu buttonup do
	(
		updatetimer_taperu.active = false
		taperu.value = 0.0
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on taperV buttondown do
	--
	--	Remarks:
	-- 		tape horizontally the selected vertices (buttondown event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on taperV buttondown do
	(
		getsetup()
		updatetimer_taperv.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on updatetimer_taperv tick do
	--
	--	Remarks:
	-- 		used to control the taperV action
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on updatetimer_taperv tick do
	(
		updatetimer_taperv.active = false
		with redraw off
		(
			for i = 1 to sel.count do
			(
				amt = (uvs[i].y / range.y) * taperv.value
				uv.setvertexposition2 currenttime sel[i] ([uvs[i].x * (1.0 + amt), uvs[i].y, uvs[i].z] + offset) false false
			)
		)
		
		uv.updateview()	
		updatetimer_taperv.active = true
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on taperV buttonup do
	--
	--	Remarks:
	-- 		tape horizontally the selected vertices (buttonup event)
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on taperV buttonup do
	(
		updatetimer_taperv.active = false
		taperv.value = 0.0
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-









--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
--
--	Rollout:
--		optionsroll rollout 
--
--	Description:
--		
--
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-	--
rollout optionsroll "Options" width:114 height:421
(
	
	--==================================================================================================================================================
	-- optionsroll Variables
	--==================================================================================================================================================
	local uv = unwrapholder.uv
	
	--==================================================================================================================================================
	-- optionsroll User Interface
	--==================================================================================================================================================
	checkbox 	custsize 			"Use Bitmap Size" 		pos:[6,3] width:102 height:16 checked:true
	spinner 	custbitmapwidth 	"Width:" 				pos:[13,24] width:96 height:16 enabled:true range:[1,9999,256] type:#integer scale:1 fieldwidth:40
	spinner 	custbitmapheight 	"Height:" 				pos:[13,45] width:96 height:16 enabled:true range:[1,9999,256] type:#integer scale:1 fieldwidth:40
	checkbox 	tilebitmap 			"Tile Bitmap?" 			pos:[4,73] width:105 height:16 checked:true
	spinner 	numtiles 			"Tiles:" 				pos:[13,94] width:96 height:16 range:[1,100,1] type:#integer scale:1 fieldwidth:40
	spinner 	brightness 			"Brightness:" 			pos:[13,115] width:96 height:16 range:[0,100,0.5] fieldwidth:40
	checkbox 	constupdate 		"Constant Update" 		pos:[6,229] width:102 height:16
	checkbox 	highlightverts 		"Highlight Selected" 	pos:[6,247] width:105 height:16
	checkbox 	hiddenedges 		"Show Hidden" 			pos:[6,278] width:102 height:16
	label 		lbl2 				"Verts." 				pos:[25,263] width:42 height:16
	label 		lbl3 				"Edges." 				pos:[24,294] width:42 height:16
	spinner 	weldthreshold 		"" 						pos:[26,357] width:63 height:16 range:[0,10,0.01]
	label 		lbl4 				"Weld Threshold:" 		pos:[16,340] width:83 height:16
	checkbox 	blendtile 			"Blend Tile To" 		pos:[6,135] width:83 height:16
	label 		lbl5 				"Background" 			pos:[24,151] width:64 height:16
	checkbox 	centretile 			"Affect Centre Tile" 	pos:[6,166] width:102 height:16
	checkbox 	centrepixel 		"Centre Pixel Snap" 	pos:[6,307] width:102 height:16
	spinner 	gridsize 			"Grid Size:" 			pos:[11,376] width:100 height:16 range:[0,1,0.01] scale:0.01
	spinner 	snapstr 			"Snap Stregth:" 		pos:[11,398] width:100 height:16 range:[0,.5,0.5] scale:0.01
	checkbox 	syncviewport 		"Synch To " 			pos:[6,197] width:102 height:16
	label 		lbl7 				"Viewport" 				pos:[25,213] width:69 height:16
	
	--==================================================================================================================================================
	-- optionsroll events
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on optionsroll open do
	--
	--	Remarks:
	-- 		update the state of the rollout when it is opened
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on optionsroll open do
	(
		custsize.checked = not uv.getusebitmapres()
		custbitmapwidth.value = uv.getrenderwidth()
		custbitmapheight.value = uv.getrenderheight()		
		constupdate.checked = uv.getconstantupdate()
		highlightverts.checked = uv.getshowselectedvertices()
		weldthreshold.value = uv.getweldthreshold()
		tilebitmap.checked = uv.gettilemap()
		numtiles.value = uv.gettilemaplimit()
		brightness.value = uv.gettilemapbrightness()
		hiddenedges.checked = uv.getdisplayhiddenedges()
		blendtile.checked = uv.getblendtiletobackground()
		centretile.checked = uv.getbrightnessaffectscentertile()
		centrepixel.checked = uv.getmidpixelsnap()
		gridsize.value = uv.getgridsize()
		snapstr.value = uv.getgridstr()
		syncviewport.checked = uv.getsyncselectionmode()
		
		custbitmapwidth.enabled = not custsize.checked
		custbitmapheight.enabled = not custsize.checked	
		
		numtiles.enabled = tilebitmap.checked			
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on blendtile changed val do
	--
	--	Remarks:
	-- 		Sets the state of the "Blend Tile To Background" option in the Display Preferences of the Advanced Options dialog. 
	--		When set to true, the tiled images are blended to the background color or black
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on blendtile changed val do
		uv.setblendtiletobackground val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on centretile changed val do
	--
	--	Remarks:
	-- 		Sets the state of the "Affect Center Tile" option in the Display Preferences of the Advanced Options dialog. When set to true, the 
	--		brightness value affects the center tile
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on centretile changed val do
		uv.setbrightnessaffectscentertile val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on centrepixel changed val do
	--
	--	Remarks:
	-- 		Sets whether mid pixels snap is used. When false, the snap is set to the bottom right corner of the pixel. When true, it snaps to the 
	--		center of the pixel
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on centrepixel changed val do
		uv.setmidpixelsnap val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on snapstr changed val do
	--
	--	Remarks:
	-- 		Sets the Grid Size
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on snapstr changed val do
		uv.setgridsize val

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on gridsize changed val do
	--
	--	Remarks:
	-- 		Sets the strength of the grid snap. At a strength of 1 the snap system has complete control and a vertex will always snap to a grid 
	--		intersection, at a strength of 0 there will be no snapping
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on gridsize changed val do
		uv.setgridstr val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on syncviewport changed val do
	--
	--	Remarks:
	-- 		Sets the Sync. to Viewport mode.
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on syncviewport changed val do
		uv.setsyncselectionmode val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on custsize changed state do
	--
	--	Remarks:
	-- 		Set to true to use the bitmap resolution, false to use the render width and height
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on custsize changed state do
	(
		uv.setusebitmapres (not state)
		
		custbitmapwidth.enabled = not state
		custbitmapheight.enabled = not state		
	)
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on custbitmapwidth changed val do
	--
	--	Remarks:
	-- 		Sets the width in pixels of the bitmap used to render to for display
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on custbitmapwidth changed val do
		uv.setrenderwidth val

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on custbitmapheight changed val do
	--
	--	Remarks:
	-- 		Sets the height in pixels of the bitmap used to render to the display
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on custbitmapheight changed val do
		uv.setrenderheight val
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on constupdate changed state do
	--
	--	Remarks:
	-- 		Sets the state of the Constant Update value. When true, the viewport is updated on every move, otherwise it is just updated on mouse up
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on constupdate changed state do
		uv.setconstantupdate state
	
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on highlightverts changed state do
	--
	--	Remarks:
	-- 		Sets whether the selected texture vertices are also displayed in the viewport
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on highlightverts changed state do
		uv.setshowselectedvertices state

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on tilebitmap changed state do
	--
	--	Remarks:
	-- 		Enable/Disable the map tiling
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on tilebitmap changed state do
	(
		uv.settilemap state
		numtiles.enabled = state
	)

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on weldthreshold changed val do
	--
	--	Remarks:
	-- 		Sets the threshold values for welds
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on weldthreshold changed val do
		uv.setweldthreshold val

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on numtiles changed val do
	--
	--	Remarks:
	-- 		Sets the map tiling limit. Corresponds to the Tile Limit value found in the Options panel and Advanced Options dialog. The value defines 
	--		how many times to repeat the map outside of the original map area
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on numtiles changed val do
		uv.settilemaplimit val

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on brightness changed val do
	--
	--	Remarks:
	-- 		Sets the Brightens of the tiled map outside of the original map area
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on brightness changed val do
		uv.settilemapbrightness val

	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		on hiddenedges changed val do
	--
	--	Remarks:
	-- 		Sets the state of the Display Hidden Edges option
	--	Parameters:
	--
	--	Return Value:
	--		none
	----------------------------------------------------------------------------------------------------------------------------------------------------
	on hiddenedges changed val do
		uv.setdisplayhiddenedges val
)	
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-



















--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-- OpenUnwrapUI MacroScript
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
macroScript OpenUnwrapUI category:"UVW Unwrap" internalcategory: "UVW Unwrap" toolTip:"OpenUnwrapUI" silentErrors:TRUE
(
	--==================================================================================================================================================
	-- OpenUnwrapUI local variables
	--==================================================================================================================================================
	local uv = modpanel.getcurrentobject()
	
	--==================================================================================================================================================
	-- OpenUnwrapUI funtions 
	--==================================================================================================================================================
	----------------------------------------------------------------------------------------------------------------------------------------------------
	--	Prototype: 
	--		function isDateOlderThan file1 file2
	--
	--	Remarks:
	-- 		compares two files specified and returns true if the first is older than the second
	--	Parameters:
	--
	--	Return Value:
	--		true if the first is older than the second, flase otherwise
	----------------------------------------------------------------------------------------------------------------------------------------------------
	function isDateOlderThan file1 file2 = 
	(
		d1 = stringstream (getfilemoddate file1)
		d2 = stringstream (getfilemoddate file2)
		
		numd1 = [0,0,0]
		numd2 = [0,0,0]
		
		numd1.x = (readdelimitedstring d1 "/") as integer
		numd1.y = (readdelimitedstring d1 "/") as integer
		numd1.z = (readdelimitedstring d1 " ") as integer
		
		numd2.x = (readdelimitedstring d2 "/") as integer
		numd2.y = (readdelimitedstring d2 "/") as integer
		numd2.z = (readdelimitedstring d2 " ") as integer	
	
		close d1
		close d2
		
		if numd2.z < numd1.z do (return false)
		if numd2.y < numd1.y do (return false)
		if numd2.x < numd1.x do (return false)
	
		return true
	)

	
	--==================================================================================================================================================
	-- OpenUnwrapUI execution: note that this code will always be execute each time the macroscript is run
	--==================================================================================================================================================

	-- check for the icon file and create the bmp file needed for the icon file
	if (getfiles (getdir #ui + "\\icons\\UVtools_24i.tga") ).count == 0 do
	(
		messagebox "You are missing you icon tga file!\nPlace UVtools_24i.tga in you ui\\icons dir." title:"Setup Error"
		return false
	)
	
	xPKW =  uv.GetWindowX()
	yPKW =  uv.GetWindowY()
	hPKW =  uv.GetWindowH()
	wPKW =  uv.GetWindowW()
	
	if (hPKW==0) then
	(
		yPKW = -50
	)  
	pPKW = Point2 xPKW (yPKW+hPKW)
	
	--	create icons that match backcolor if needed
	if (getfiles (getdir #ui + "\\icons\\UVtools_24i.bmp")).count == 0 or (isDateOlderThan (getdir #ui + "\\icons\\UVtools_24i.tga") (getdir #ui + "\\icons\\UVtools_24i.bmp")) == false do
	(
		backcol = (colorman.getcolor #background) * 255.0
		bmp = openbitmap (getdir #ui + "\\icons\\UVtools_24i.tga")
		icon = bitmap bmp.width bmp.height
		
		for i = 1 to bmp.height do
		(
			line = getpixels bmp [0,i - 1] bmp.width
		
			for n = 1 to bmp.width do
			(
				line[n] = backcol + ((line[n] - backcol) * (line[n].a * 0.00392157))
			)
		
			setpixels icon [0,i - 1] line
		)
		icon.filename = (getdir #ui + "\\icons\\UVtools_24i.bmp")
		save icon
		close bmp
		close icon
		icon = undefined
		bmp = undefined
		line = undefined
		gc()
		print "New unwrap icons built"
	)
		
	--switch all icons ON!
	for i = 1 to 30 do uv.showicon i true
	uv.savecurrentsettingsasdefault()

	case dock of
	(
		#right: createdialog unwrapholder pos:[xpkw + wpkw, ypkw] style:#(#style_border)
		#left: createdialog unwrapholder pos:[xpkw - 135, ypkw] style:#(#style_border)
		undefined: createdialog unwrapholder pos:[xpkw + wpkw, ypkw] style:#(#style_border)
	)

	-- invlude all the rollouts of the unwrap tools
	addsubrollout unwrapholder.holder aligntools
	addsubrollout unwrapholder.holder spacetools
	addsubrollout unwrapholder.holder transformtools
	addsubrollout unwrapholder.holder deformtools
	addsubrollout unwrapholder.holder dumptools
	addsubrollout unwrapholder.holder relaxtools
	addsubrollout unwrapholder.holder centretools rolledup:true
	addsubrollout unwrapholder.holder tiletools rolledup:true
	addsubrollout unwrapholder.holder indextools rolledup:true
	addsubrollout unwrapholder.holder softSelTools rolledup:true
	addsubrollout unwrapholder.holder moretools rolledup:true 
	addsubrollout unwrapholder.holder optionsroll rolledup:true
 
	if (hPKW==0) then
	(
		unwrapholder.height = 0
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-











--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-- Initialize MoveUnwrapUI 
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
macroScript MoveUnwrapUI category:"UVW Unwrap" internalcategory: "UVW Unwrap"  toolTip:"MoveUnwrapUI" silentErrors:TRUE
(
	local uv = modpanel.getcurrentobject()
	
	xPKW = uv.GetWindowX()
	yPKW = uv.GetWindowY()
	hPKW = uv.GetWindowH()
	wPKW =  uv.GetWindowW() 
	
	if (hPKW==0) then  
	(
		yPKW = -50
	)
	
	case dock of
	(
		#right: SetDialogPos unwrapholder [xpkw + wpkw, ypkw]	
		#left: SetDialogPos unwrapholder [xpkw - 135, ypkw]
		undefined: SetDialogPos unwrapholder [xpkw + wpkw, ypkw]
	)
	
	if (hPKW==0) then
	(
		unwrapholder.height = 0
	)
	else
	(
		if unwrapholder.hidestate then
		(
			unwrapholder.height = 20
		)
		else
		(
			unwrapholder.updateUI()
			/*height = hPKW - 6
			unwrapholder.holder.height = hPKW - 100 			
			unwrapholder.subObjVert.pos = [21,(hPKW - 73)]
			unwrapholder.subObjEdge.pos = [51,(hPKW - 73)]
			unwrapholder.subObjFace.pos = [81,(hPKW - 73)]
			unwrapholder.syncelement.pos = [12,(hPKW - 42)]
			unwrapholder.softcel.pos = [12,(hPKW - 22)]*/
		)
	)
)
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-



--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-- max9 addittions
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-

macroScript UVWUnwrap_Stitch 
enabledIn:#("max", "viz", "vizr") 
	category:"UVW Unwrap"
	internalcategory: "UVW Unwrap"  --do not localize
	toolTip:"Stitch Selected"
	buttontext:"Stitch Selected"
(
	on execute do actionMan.executeAction 2077580866 "40043"  -- All Commands: Stitch
	on altExecute type do actionMan.executeAction 2077580866 "40044"  -- All Commands: Stitch Dialog	
)

macroScript UVWUnwrap_Sketch
enabledIn:#("max", "viz", "vizr") 
	category:"UVW Unwrap"
	internalcategory: "UVW Unwrap"  --do not localize
	toolTip:"Sketch Vertices"
	buttontext:"Sketch Vertices"
(
	on execute do actionMan.executeAction 2077580866 "40114"  -- All Commands: Sketch
	on altExecute type do actionMan.executeAction 2077580866 "40112"  -- All Commands: Sketch Dialog
)

macroScript UVWUnwrap_Pack
enabledIn:#("max", "viz", "vizr") 
	category:"UVW Unwrap"
	internalcategory: "UVW Unwrap"  --do not localize
	toolTip:"Pack UVs"
	buttontext:"Pack UVs"
(
	on execute do actionMan.executeAction 2077580866 "40074"  -- All Commands: Pack
	on altExecute type do actionMan.executeAction 2077580866 "40075"  -- All Commands: Pack Dialog
)

macroScript UVWUnwrap_Relax
enabledIn:#("max", "viz", "vizr") 
	category:"UVW Unwrap"
	internalcategory: "UVW Unwrap"  --do not localize
	toolTip:"Relax"
	buttontext:"Relax"
(
	on execute do actionMan.executeAction 2077580866 "40135"  -- All Commands: Relax
	on altExecute type do actionMan.executeAction 2077580866 "40136"  -- All Commands: Relax Dialog
)









--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
-- Initialize CloseUnwrapUI 
--=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-=-
macroScript CloseUnwrapUI category:"UVW Unwrap" internalcategory: "UVW Unwrap" toolTip:"CloseUnwrapUI" silentErrors:TRUE	
(
	destroydialog unwrapholder
)
	